@@ -21,7 +21,7 @@
 #ifndef _MUESLI_CUSTOM_
 #define _MUESLI_CUSTOM_
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 
@@ -155,8 +155,8 @@ muesli_value_t result = (custom_interface->handle_option)(custom_interface->app_
 							  "custom");
 
     switch (result.type) {
-    case muesli_value_string:
-      custom_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      custom_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_float:
       custom_return_number(cs, result.data.as_float);
@@ -191,8 +191,8 @@ custom_get_parameters(custom_state *cs)
 							      "custom");
 
     switch (result.type) {
-    case muesli_value_string:
-      custom_set_table_string(cs, table, (char*)(option_names->name), result.data.as_string);
+    case muesli_value_const_string:
+      custom_set_table_string(cs, table, (char*)(option_names->name), result.data.as_const_string);
       break;
     case muesli_value_float:
       custom_set_table_number(cs, table, (char*)(option_names->name), result.data.as_float);
@@ -240,8 +240,9 @@ muesli_custom_eval_in_language(custom_state *cs)
     case muesli_value_integer:
       custom_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      custom_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+    case muesli_value_malloced_string:
+      custom_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       custom_return_boolean(cs, result.data.as_bool);
@@ -283,8 +284,9 @@ custom_eval_function(custom_state *cs)
     case muesli_value_integer:
       custom_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      custom_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+    case muesli_value_malloced_string:
+      custom_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       custom_return_boolean(cs, result.data.as_int);
@@ -354,6 +356,7 @@ custom_load_file(evaluator_interface *interface,
 		const char *filename)
 {
   int muesli_flags = interface->flags;
+  Muesli_Enter_Evaluator(interface, 0);
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loading %s\n", filename);
   }
@@ -361,6 +364,7 @@ custom_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
+  Muesli_Leave_Evaluator();
 }
 
 static muesli_value_t
@@ -370,11 +374,10 @@ custom_eval_string(evaluator_interface *evaluator,
 		   int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(evaluator, transient);
   ANULL_VALUE(result);
 
   if (custom_c_string) {
-    int old_ambient_transient = ambient_transient;
-    ambient_transient = transient;
     // fprintf(stderr, "Custom evaluating string \"%s\"\n", custom_c_string);
 
     // Fill in: evaluate custom_c_string, decide what kind of result
@@ -383,9 +386,10 @@ custom_eval_string(evaluator_interface *evaluator,
 
 
 
-    ambient_transient = old_ambient_transient;
   }
 
+  Muesli_Leave_Evaluator();
+
   return result;
 }
 
@@ -394,10 +398,13 @@ custom_eval_0(struct evaluator_interface *interface,
 	      const char *scratch,
 	      unsigned int length)
 {
-  muesli_value_t result = custom_eval_string(interface,
-					     scratch,
-					     length,
-					     0);
+  muesli_value_t result;
+  Muesli_Enter_Evaluator(interface, 0);
+  result = custom_eval_string(interface,
+			      scratch,
+			      length,
+			      0);
+  Muesli_Leave_Evaluator();
   switch (result.type) {
   case muesli_value_float:
     return result.data.as_float;
