@@ -21,7 +21,7 @@
 #ifndef _MUESLI_ERLANG_
 #define _MUESLI_ERLANG_
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 
@@ -156,8 +156,8 @@ muesli_value_t result = (erlang_interface->handle_option)(erlang_interface->app_
 							  "erlang");
 
     switch (result.type) {
-    case muesli_value_string:
-      erlang_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      erlang_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_float:
       erlang_return_number(cs, result.data.as_float);
@@ -192,8 +192,8 @@ erlang_get_parameters()
 							      "erlang");
 
     switch (result.type) {
-    case muesli_value_string:
-      erlang_set_table_string(cs, table, (char*)(option_names->name), result.data.as_string);
+    case muesli_value_const_string:
+      erlang_set_table_string(cs, table, (char*)(option_names->name), result.data.as_const_string);
       break;
     case muesli_value_float:
       erlang_set_table_number(cs, table, (char*)(option_names->name), result.data.as_float);
@@ -243,8 +243,8 @@ muesli_erlang_eval_in_language()
     case muesli_value_integer:
       erlang_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      erlang_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      erlang_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       erlang_return_boolean(cs, result.data.as_bool);
@@ -283,8 +283,8 @@ erlang_eval_custom_function()
     case muesli_value_integer:
       erlang_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      erlang_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      erlang_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       erlang_return_boolean(cs, result.data.as_int);
@@ -356,6 +356,7 @@ erlang_load_file(evaluator_interface *interface,
 		const char *filename)
 {
   int muesli_flags = interface->flags;
+  Muesli_Enter_Evaluator(interface, transient);
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loading %s\n", filename);
   }
@@ -363,6 +364,7 @@ erlang_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
+  Muesli_Leave_Evaluator();
 }
 
 static muesli_value_t
@@ -372,11 +374,10 @@ erlang_eval_string(evaluator_interface *evaluator,
 		   int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(evaluator, transient);
   ANULL_VALUE(result);
 
   if (erlang_c_string) {
-    int old_ambient_transient = ambient_transient;
-    ambient_transient = transient;
     // fprintf(stderr, "Erlang evaluating string \"%s\"\n", erlang_c_string);
 
     // Fill in: evaluate erlang_c_string, decide what kind of result
@@ -385,9 +386,8 @@ erlang_eval_string(evaluator_interface *evaluator,
 
 
 
-    ambient_transient = old_ambient_transient;
   }
-
+  Muesli_Leave_Evaluator();
   return result;
 }
 
