@@ -20,7 +20,7 @@
 #ifndef _MUESLI_EXEC_
 #define _MUESLI_EXEC_
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 
@@ -76,6 +76,8 @@ exec_load_file(evaluator_interface *interface,
 
 #define RESULT_MAX 1024
 
+static int ambient_transient = 0;
+
 static muesli_value_t
 exec_eval_string(evaluator_interface *interface,
 		 const char *exec_c_string,
@@ -87,6 +89,7 @@ exec_eval_string(evaluator_interface *interface,
   int tracing = (interface->flags & TRACE_MUESLI_CHILD_PROCESS);
 
   muesli_value_t result;
+  Muesli_Enter_Evaluator(interface, transient);
   ANULL_VALUE(result);
 
   muesli_split_args(exec_c_string, interface->underlying_program, &argv, &argc);
@@ -164,8 +167,8 @@ exec_eval_string(evaluator_interface *interface,
 	return result;
       } else {
 	result_buf[RESULT_MAX - 1] = '\0';
-	result.data.as_string = muesli_malloc_copy_string(result_buf);
-	result.type = muesli_value_string;
+	result.data.as_malloced_string = muesli_malloc_copy_string(result_buf);
+	result.type = muesli_value_malloced_string;
 	close(pfd[0]);
 	close(pfd[1]);
 	return result;
@@ -174,6 +177,7 @@ exec_eval_string(evaluator_interface *interface,
     close(pfd[0]);
     close(pfd[1]);
   }
+  Muesli_Leave_Evaluator();
   return result;
 }
 
