@@ -21,7 +21,7 @@
 #ifndef _MUESLI_GS_
 #define _MUESLI_GS_
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 
@@ -126,8 +126,8 @@ muesli_value_t result = (gs_interface->handle_option)(gs_interface->app_params,
 							  "gs");
 
     switch (result.type) {
-    case muesli_value_string:
-      gs_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      gs_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_float:
       gs_return_number(cs, result.data.as_float);
@@ -162,8 +162,8 @@ gs_get_parameters(gs_state *cs)
 							      "gs");
 
     switch (result.type) {
-    case muesli_value_string:
-      gs_set_table_string(cs, table, (char*)(option_names->name), result.data.as_string);
+    case muesli_value_const_string:
+      gs_set_table_string(cs, table, (char*)(option_names->name), result.data.as_const_string);
       break;
     case muesli_value_float:
       gs_set_table_number(cs, table, (char*)(option_names->name), result.data.as_float);
@@ -213,8 +213,8 @@ muesli_gs_eval_in_language(gs_state *cs)
     case muesli_value_integer:
       gs_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      gs_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      gs_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       gs_return_boolean(cs, result.data.as_bool);
@@ -253,8 +253,8 @@ gs_eval_custom_function(gs_state *cs)
     case muesli_value_integer:
       gs_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      gs_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      gs_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       gs_return_boolean(cs, result.data.as_int);
@@ -331,6 +331,7 @@ gs_load_file(evaluator_interface *interface,
 		const char *filename)
 {
   int muesli_flags = interface->flags;
+  Muesli_Enter_Evaluator(interface, 0);
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loading %s\n", filename);
   }
@@ -338,6 +339,7 @@ gs_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
+  Muesli_Leave_Evaluator();
 }
 
 static muesli_value_t
@@ -347,6 +349,7 @@ gs_eval_string(evaluator_interface *evaluator,
 		   int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(evaluator, transient);
   ANULL_VALUE(result);
 
   if (gs_c_string) {
@@ -362,7 +365,7 @@ gs_eval_string(evaluator_interface *evaluator,
 
     ambient_transient = old_ambient_transient;
   }
-
+  Muesli_Leave_Evaluator();
   return result;
 }
 
