@@ -21,7 +21,7 @@
 #ifndef _MUESLI_GUILE_
 #define _MUESLI_GUILE_
 
-#include "../config.h"
+#include "../muesli-config.h"
 
 #ifdef HAVE_LIBGUILE
 
@@ -57,7 +57,7 @@ muesli_guile_eval_string(evaluator_interface *interface,
 			int transient);
 
 // Interface from the rest of Muesli, set each time Muesli calls us
-static evaluator_interface *current_interface = NULL;
+static evaluator_interface *guile_interface = NULL;
 
 static int ambient_transient = 0;
 
@@ -70,7 +70,7 @@ static char number_buf[NUMBER_BUF_SIZE];
 /* Value conversion */
 /********************/
 
-static muesli_value_t
+muesli_value_t
 guile_to_muesli(SCM guile_value)
 {
   muesli_value_t muesli_value;
@@ -91,24 +91,31 @@ guile_to_muesli(SCM guile_value)
        / (float)scm_to_double(scm_denominator(guile_value)));
     muesli_value.type = muesli_value_float; 
   } else if (scm_is_symbol(guile_value)) {
-    muesli_value.data.as_string = scm_to_locale_string(scm_symbol_to_string(guile_value));
-    muesli_value.type = muesli_value_string;
+    muesli_value.data.as_const_string = scm_to_locale_string(scm_symbol_to_string(guile_value));
+    muesli_value.type = muesli_value_const_string;
   } else if (scm_is_string(guile_value)) {
-    muesli_value.data.as_string = scm_to_locale_string(guile_value);
-    muesli_value.type = muesli_value_string;
+    muesli_value.data.as_const_string = scm_to_locale_string(guile_value);
+    muesli_value.type = muesli_value_const_string;
   }
 
   return muesli_value;
 }
 
-static SCM
+SCM
 muesli_to_guile(muesli_value_t muesli_value)
 {
   switch (muesli_value.type) {
-  case muesli_value_string:
-    if (muesli_value.data.as_string != NULL) {
-      return scm_from_locale_stringn(muesli_value.data.as_string,
-				     strlen(muesli_value.data.as_string));
+  case muesli_value_const_string:
+    if (muesli_value.data.as_const_string != NULL) {
+      return scm_from_locale_stringn(muesli_value.data.as_const_string,
+				     strlen(muesli_value.data.as_const_string));
+    } else {
+      return SCM_UNDEFINED;
+    }
+  case muesli_value_malloced_string:
+    if (muesli_value.data.as_malloced_string != NULL) {
+      return scm_from_locale_stringn(muesli_value.data.as_malloced_string,
+				     strlen(muesli_value.data.as_malloced_string));
     } else {
       return SCM_UNDEFINED;
     }
@@ -136,26 +143,26 @@ muesli_guile_set_parameter(SCM option, SCM value)
 
   if (scm_is_string(value)) {
     char *option_value = scm_to_locale_string(value);
-    ok = muesli_set_parameter(current_interface,
-			      current_interface->getopt_options,
+    ok = muesli_set_parameter(guile_interface,
+			      guile_interface->getopt_options,
 			      option_name, option_value,
 			      (const char*)"guile");
     free(option_value);
   } else if (scm_is_real(value)) {
     snprintf(number_buf, NUMBER_BUF_SIZE, "%f", (float)scm_to_double(value));
-    ok = muesli_set_parameter(current_interface,
-			      current_interface->getopt_options,
+    ok = muesli_set_parameter(guile_interface,
+			      guile_interface->getopt_options,
 			      option_name, number_buf,
 			      (const char*)"guile");
   } else if (scm_is_integer(value)) {
     snprintf(number_buf, NUMBER_BUF_SIZE, "%d", scm_to_int(value));
-    ok = muesli_set_parameter(current_interface,
-			      current_interface->getopt_options,
+    ok = muesli_set_parameter(guile_interface,
+			      guile_interface->getopt_options,
 			      option_name, number_buf,
 			      (const char*)"guile");
   } else {
-    ok = muesli_set_parameter(current_interface,
-			      current_interface->getopt_options,
+    ok = muesli_set_parameter(guile_interface,
+			      guile_interface->getopt_options,
 			      option_name, (char*)"true",
 			      (const char*)"guile");
   }
@@ -182,12 +189,12 @@ static SCM
 muesli_guile_get_parameter(SCM option)
 {
   char *option_name = scm_to_locale_string(option);
-  char option_code = muesli_find_option_letter(current_interface->getopt_options, option_name);
+  char option_code = muesli_find_option_letter(guile_interface->getopt_options, option_name);
   free(option_name);
 
   if (option_code != -1) {
-    return muesli_to_guile((current_interface->handle_option)
-			   (current_interface->app_params,
+    return muesli_to_guile((guile_interface->handle_option)
+			   (guile_interface->app_params,
 			    option_code,	// option
 			    NULL, 0.0,	// value
 			    0,	// set
@@ -208,7 +215,7 @@ muesli_guile_get_parameters()
       SCM pair = scm_car(names);
       SCM lisp_result;
 
-      muesli_value_t result = (current_interface->handle_option)(current_interface->app_params,
+      muesli_value_t result = (guile_interface->handle_option)(guile_interface->app_params,
 								 (int)(scm_to_int(scm_cdr(pair))), // opt
 								 NULL, 0.0,		     // value
 								 0,		     // set
@@ -344,27 +351,27 @@ guile_evaluator_inner_init(evaluator_interface *interface)
   struct option *option_names;
   scm_init_guile();
 
-  current_interface = interface;
+  guile_interface = interface;
 
   // general functions:
   if (muesli_extensions & MUESLI_EXTENSION_SINGLE_PARAMETER) {
-    Muesli_Add_Fn_1(current_interface, (char*)"get-parameter", muesli_guile_get_parameter);
-    Muesli_Add_Fn_2(current_interface, (char*)"set-parameter", muesli_guile_set_parameter);
+    Muesli_Add_Fn_1(guile_interface, (char*)"get-parameter", muesli_guile_get_parameter);
+    Muesli_Add_Fn_2(guile_interface, (char*)"set-parameter", muesli_guile_set_parameter);
   }
   if (muesli_extensions & MUESLI_EXTENSION_PARAMETER_BLOCK) {
-    Muesli_Add_Fn_0(current_interface, (char*)"parameters", muesli_guile_get_parameters);
-    Muesli_Add_Fn_1(current_interface, (char*)"modify-parameters", muesli_guile_set_parameters);
+    Muesli_Add_Fn_0(guile_interface, (char*)"parameters", muesli_guile_get_parameters);
+    Muesli_Add_Fn_1(guile_interface, (char*)"modify-parameters", muesli_guile_set_parameters);
   }
   if (muesli_extensions & MUESLI_EXTENSION_EVAL_IN_LANGUAGE) {
-    Muesli_Add_Fn_2(current_interface, (char*)"eval-in-language", muesli_guile_eval_in_language);
+    Muesli_Add_Fn_2(guile_interface, (char*)"eval-in-language", muesli_guile_eval_in_language);
   }
   if (muesli_extensions & MUESLI_EXTENSION_CUSTOM) {
-    Muesli_Add_Fn_1(current_interface, (char*)"custom", muesli_guile_custom_eval_function);
-    Muesli_Add_Fn_V(current_interface, (char*)"bytecode", muesli_guile_eval_bytecode_string);
+    Muesli_Add_Fn_1(guile_interface, (char*)"custom", muesli_guile_custom_eval_function);
+    Muesli_Add_Fn_V(guile_interface, (char*)"bytecode", muesli_guile_eval_bytecode_string);
   }
 
   // set up option names
-  for (option_names = current_interface->getopt_options;
+  for (option_names = guile_interface->getopt_options;
        (option_names != NULL) && (option_names->name != 0);
        option_names++) {
     option_names_table = scm_cons(scm_cons(scm_from_locale_symbol((char*)(option_names->name)),
@@ -404,6 +411,7 @@ guile_add_function(evaluator_interface *interface,
 		  int arity, const char *name,
 		  void *function)
 {
+  // fprintf(stderr, "guile defining %s(%d)\n", name, arity);
   switch (arity) {
   case ARGS_COUNT_VARIABLE:
     scm_c_define_gsubr(name, 0, 0, 1, function);
@@ -435,8 +443,8 @@ guile_load_file(evaluator_interface *interface,
 	       const char *filename)
 {
   int muesli_flags = interface->flags;
-
-  current_interface = interface;
+  Muesli_Enter_Evaluator(interface, 0);
+  guile_interface = interface;
 
   
   if (muesli_flags & TRACE_MUESLI_LOAD) {
@@ -448,7 +456,7 @@ guile_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
-  
+  Muesli_Leave_Evaluator();  
 }
 
 ////////////////
@@ -474,33 +482,34 @@ muesli_guile_eval_string(evaluator_interface *interface,
 			unsigned int string_length,
 			int transient)
 {
-  int old_ambient_transient = ambient_transient;
   muesli_value_t result;
-  
-  ambient_transient = transient;
-  current_interface = interface;
+  Muesli_Enter_Evaluator(interface, transient);  
+
+  guile_interface = interface;
 
   result = guile_to_muesli(scm_c_eval_string(scratch));
 
-  ambient_transient = old_ambient_transient;
-
+  Muesli_Leave_Evaluator();
   return result;
 }
 
 void
 guile_setup(evaluator_interface *new_guile_interface)
 {
-  current_interface = new_guile_interface;
+  guile_interface = new_guile_interface;
 
-  current_interface->eval_string = muesli_guile_eval_string;
-  current_interface->add_function = guile_add_function;
-  current_interface->load_file = guile_load_file;
-  current_interface->array_separator = (char*)" ";
+  guile_interface->eval_string = muesli_guile_eval_string;
+  guile_interface->add_function = guile_add_function;
+  guile_interface->load_file = guile_load_file;
+  guile_interface->array_separator = (char*)" ";
 
-  current_interface->from_muesli_value = muesli_to_guile;
-  current_interface->to_muesli_value = guile_to_muesli;
+  guile_interface->from_muesli_value = (from_muesli_value_t)muesli_to_guile;
+  guile_interface->to_muesli_value = (to_muesli_value_t)guile_to_muesli;
 
-  current_interface->version = scm_to_locale_string(scm_version());
+#if 0
+  /* crashes when running within emacs */
+  guile_interface->version = scm_to_locale_string(scm_version());
+#endif
 }
 
 #endif
