@@ -20,7 +20,7 @@
 #ifndef _MUESLI_LUA_
 #define _MUESLI_LUA_
 
-#include "../config.h"
+#include "../muesli-config.h"
 
 #ifdef HAVE_LIBLUA
 
@@ -84,8 +84,11 @@ int
 muesli_to_lua_push(lua_State *L, muesli_value_t mval)
 {
   switch (mval.type) {
-  case muesli_value_string:
-    lua_pushstring(L, mval.data.as_string);
+  case muesli_value_const_string:
+    lua_pushstring(L, mval.data.as_const_string);
+    break;
+  case muesli_value_malloced_string:
+    lua_pushstring(L, mval.data.as_malloced_string);
     break;
   case muesli_value_float:
     lua_pushnumber(L, mval.data.as_float);
@@ -307,6 +310,7 @@ lua_load_file(evaluator_interface *interface,
 {
   int muesli_flags = interface->flags;
   lua_State *lua_state = (lua_State *)interface->state;
+  Muesli_Enter_Evaluator(interface, 0);
 
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loading %s\n", filename);
@@ -321,6 +325,7 @@ lua_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
+  Muesli_Leave_Evaluator();
 }
 
 /*******************/
@@ -335,13 +340,12 @@ lua_eval_string(evaluator_interface *interface,
 {
   lua_State *lua_state = (lua_State *)interface->state;
   muesli_value_t result;
+  Muesli_Enter_Evaluator(interface, transient);
   ANULL_VALUE(result);
 
   lua_clear_stack();
 
   if (lua_c_string) {
-    int old_ambient_transient = ambient_transient;
-    ambient_transient = transient;
     // fprintf(stderr, "Lua evaluating string \"%s\"\n", lua_c_string);
     if (luaL_loadbuffer(lua_state,
 			lua_c_string,
@@ -350,13 +354,10 @@ lua_eval_string(evaluator_interface *interface,
 	|| lua_pcall(lua_state,0,1,0)) {
       // Lua error compiling buffer
       fprintf(stderr, "%s\n", lua_tostring(lua_state,-1));
-      ambient_transient = old_ambient_transient;
     }
 
     int stack_count = lua_gettop(lua_state);
 
-    ambient_transient = old_ambient_transient;
-
     if (stack_count >= 1) {
       switch (lua_type (lua_state, -1)) {
       case LUA_TNUMBER:
@@ -372,8 +373,8 @@ lua_eval_string(evaluator_interface *interface,
 	result.type = muesli_value_boolean;
 	break;
       case LUA_TSTRING:
-	result.data.as_string = lua_tostring(lua_state, -1);
-	result.type = muesli_value_string;
+	result.data.as_const_string = lua_tostring(lua_state, -1);
+	result.type = muesli_value_const_string;
 	break;
       default:
 	// I don't think we need fill in the result here
@@ -382,6 +383,8 @@ lua_eval_string(evaluator_interface *interface,
     }
   }
 
+  Muesli_Leave_Evaluator();
+
   return result;
 }
 
