@@ -22,7 +22,7 @@
 #define _MUESLI_MACHINE_CODE_
 
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 #include <stdlib.h>
@@ -73,16 +73,20 @@ machine_code_eval_string_no_arg(evaluator_interface *interface,
 			   interface->app_data);
 }
 
+static int ambient_transient = 0;
+
 static muesli_value_t
 machine_code_eval_string(evaluator_interface *interface,
 			 const char *scratch, unsigned int length,
 			 int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(interface, transient);
   result.type = muesli_value_float;
   result.data.as_float = machine_code_eval((machine_code_state_t*)interface->state,
 					   scratch, length,
 					   interface->app_data);
+  Muesli_Leave_Evaluator();
   return result;
 }
 
