@@ -22,7 +22,7 @@
 #define _MUESLI_OCTAVE_CPP_
 
 extern "C" {
-#include "../config.h"
+#include "../muesli-config.h"
 }
 
 #ifdef HAVE_LIBOCTINTERP
@@ -269,8 +269,9 @@ Get parameter @var{name}.\n\
       case muesli_value_integer:
 	retval = octave_value(result.data.as_int);
 	break;
-      case muesli_value_string:
-	retval = octave_value(result.data.as_string);
+      case muesli_value_const_string:
+      case muesli_value_malloced_string:
+	retval = octave_value(result.data.as_const_string);
 	break;
       case muesli_value_boolean:
 	retval = octave_value(result.data.as_bool);
@@ -313,8 +314,9 @@ Return current parameters as a map.\n\
 							      "octave");
 
     switch (result.type) {
-    case muesli_value_string:
-      map.assign(option_names->name, octave_value(result.data.as_string));
+    case muesli_value_const_string:
+    case muesli_value_malloced_string:
+      map.assign(option_names->name, octave_value(result.data.as_const_string));
       break;
     case muesli_value_float:
       map.assign(option_names->name, octave_value(result.data.as_float));
@@ -371,8 +373,9 @@ Evaluate @var{fragment} using the evaluator for @var{language}.\n\
     case muesli_value_integer:
       retval = octave_value(result.data.as_int);
       break;
-    case muesli_value_string:
-      retval = octave_value(result.data.as_string);
+    case muesli_value_const_string:
+    case muesli_value_malloced_string:
+      retval = octave_value(result.data.as_const_string);
       break;
     case muesli_value_boolean:
       retval = octave_value(result.data.as_int);
@@ -417,8 +420,9 @@ Evaluate @var{x} using the custom evaluator.\n\
     case muesli_value_integer:
       retval = octave_value(result.data.as_int);
       break;
-    case muesli_value_string:
-      retval = octave_value(result.data.as_string);
+    case muesli_value_const_string:
+    case muesli_value_malloced_string:
+      retval = octave_value(result.data.as_const_string);
       break;
     case muesli_value_boolean:
       retval = octave_value(result.data.as_int);
@@ -508,11 +512,14 @@ Return current parameters as a map.\n\
   set_global_value("options", octave_value(option_names_table));
 }
 
+static int ambient_transient = 0;
+
 void
 octave_load_file(evaluator_interface *interface,
 		const char *filename)
 {
   int muesli_flags = interface->flags;
+  Muesli_Enter_Evaluator(interface, 0);
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loading %s\n", filename);
   }
@@ -522,6 +529,7 @@ octave_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
+  Muesli_Leave_Evaluator();
 }
 
 muesli_value_t
@@ -621,8 +629,8 @@ octave_eval_string(evaluator_interface *evaluator,
 	result.data.as_bool = res.bool_value();
 	result.type = muesli_value_boolean;
       } else if (res.is_string()) {
-	result.data.as_string = res.string_value().c_str();
-	result.type = muesli_value_string;
+	result.data.as_const_string = res.string_value().c_str();
+	result.type = muesli_value_const_string;
       } else {
 	result.type = muesli_value_none;
       }
