@@ -22,7 +22,7 @@
 #define _MUESLI_PIPE_
 
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 
@@ -255,11 +255,14 @@ pipe_evaluator_shutdown(evaluator_interface *interface)
 #endif
 }
 
+static int ambient_transient = 0; 
+
 static void
 pipe_load_file(evaluator_interface *interface,
 	       const char* filename)
 {
   int muesli_flags = interface->flags;
+  Muesli_Enter_Evaluator(interface, 0);
 
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loading %s\n", filename);
@@ -271,6 +274,7 @@ pipe_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
+  Muesli_Leave_Evaluator();
 }
 
 static muesli_value_t
@@ -280,6 +284,7 @@ pipe_eval_string(evaluator_interface *interface,
 		 int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(interface, transient);
   ANULL_VALUE(result);
 
   if (interface->flags & TRACE_MUESLI_CHILD_PROCESS) {
@@ -299,6 +304,7 @@ pipe_eval_string(evaluator_interface *interface,
     }
   }
 
+  Muesli_Leave_Evaluator();
   return result;
 }
 
