@@ -21,7 +21,7 @@
 #ifndef _MUESLI_PROLOG_
 #define _MUESLI_PROLOG_
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 
@@ -158,8 +158,8 @@ muesli_value_t result = (prolog_interface->handle_option)(prolog_interface->app_
 							  "prolog");
 
     switch (result.type) {
-    case muesli_value_string:
-      prolog_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      prolog_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_float:
       prolog_return_number(cs, result.data.as_float);
@@ -194,8 +194,8 @@ prolog_get_parameters(prolog_state *cs)
 							      "prolog");
 
     switch (result.type) {
-    case muesli_value_string:
-      prolog_set_table_string(cs, table, (char*)(option_names->name), result.data.as_string);
+    case muesli_value_const_string:
+      prolog_set_table_string(cs, table, (char*)(option_names->name), result.data.as_const_string);
       break;
     case muesli_value_float:
       prolog_set_table_number(cs, table, (char*)(option_names->name), result.data.as_float);
@@ -245,8 +245,8 @@ muesli_prolog_eval_in_language(prolog_state *cs)
     case muesli_value_integer:
       prolog_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      prolog_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      prolog_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       prolog_return_boolean(cs, result.data.as_bool);
@@ -285,8 +285,8 @@ prolog_eval_custom_function(prolog_state *cs)
     case muesli_value_integer:
       prolog_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      prolog_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      prolog_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       prolog_return_boolean(cs, result.data.as_int);
@@ -373,6 +373,7 @@ prolog_load_file(evaluator_interface *interface,
   int muesli_flags = interface->flags;
   int buff_len = strlen((char*)"consult([''])") + strlen(filename) + 1;
   char *buff = (char*)alloca(buff_len);
+  Muesli_Enter_Evaluator(interface, 0);
 
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loading %s\n", filename);
@@ -416,6 +417,7 @@ prolog_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
+  Muesli_Leave_Evaluator();
 }
 
 static muesli_value_t
@@ -425,6 +427,7 @@ prolog_eval_string(evaluator_interface *evaluator,
 		   int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(evaluator, transient);
   ANULL_VALUE(result);
 
   if (prolog_c_string) {
@@ -438,14 +441,14 @@ prolog_eval_string(evaluator_interface *evaluator,
       Alternatively, look in the sources to find how this is
       implemented:
 
-       -g goal
+      -g goal
 
-              Goal is executed just before entering the top level. The
-              default is a predicate which prints the welcome
-              message. The welcome message can thus be suppressed by
-              giving -g true.  goal can be a complex term. In this
-              case, quotes are normally needed to protect it from
-              being expanded by the Unix shell.
+      Goal is executed just before entering the top level. The
+      default is a predicate which prints the welcome
+      message. The welcome message can thus be suppressed by
+      giving -g true.  goal can be a complex term. In this
+      case, quotes are normally needed to protect it from
+      being expanded by the Unix shell.
     */
 
 
@@ -513,8 +516,8 @@ prolog_eval_string(evaluator_interface *evaluator,
 	{
 	  char *chars;
 	  PL_get_atom_chars(a2, &chars);
-	  result.data.as_string = muesli_malloc_copy_string(chars);
-	  result.type = muesli_value_string;
+	  result.data.as_malloced_string = muesli_malloc_copy_string(chars);
+	  result.type = muesli_value_malloced_string;
 	}
 	break;
       case PL_STRING:
@@ -522,16 +525,16 @@ prolog_eval_string(evaluator_interface *evaluator,
 	  char *chars;
 	  unsigned int size;
 	  PL_get_string_chars(a2, &chars, &size);
-	  result.data.as_string = muesli_malloc_copy_string(chars);
-	  result.type = muesli_value_string;
+	  result.data.as_malloced_string = muesli_malloc_copy_string(chars);
+	  result.type = muesli_value_malloced_string;
 	}
 	break;
       default:
 	{
 	  char *chars;
 	  PL_get_chars(a2, &chars, CVT_ALL);
-	  result.data.as_string = muesli_malloc_copy_string(chars);
-	  result.type = muesli_value_string;
+	  result.data.as_malloced_string = muesli_malloc_copy_string(chars);
+	  result.type = muesli_value_malloced_string;
 	}
       }
     }
@@ -541,6 +544,7 @@ prolog_eval_string(evaluator_interface *evaluator,
     ambient_transient = old_ambient_transient;
   }
 
+  Muesli_Leave_Evaluator();
   return result;
 }
 
