@@ -21,7 +21,7 @@
 #ifndef _MUESLI_RUBY_
 #define _MUESLI_RUBY_
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 
@@ -74,8 +74,8 @@ ruby_to_muesli(VALUE rubyval)
   case T_STRING:
     {
       char *s = StringValueCStr(rubyval);
-      mval.data.as_string = muesli_malloc_copy_string(s);
-      mval.type = muesli_value_string;
+      mval.data.as_malloced_string_string = muesli_malloc_copy_string(s);
+      mval.type = muesli_value_malloced_string_string;
     }
     break;
   case T_SYMBOL:
@@ -169,8 +169,8 @@ muesli_value_t result = (ruby_interface->handle_option)(ruby_interface->app_para
 							  "ruby");
 
     switch (result.type) {
-    case muesli_value_string:
-      ruby_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      ruby_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_float:
       ruby_return_number(cs, result.data.as_float);
@@ -205,8 +205,8 @@ ruby_get_parameters()
 							      "ruby");
 
     switch (result.type) {
-    case muesli_value_string:
-      ruby_set_table_string(cs, table, (char*)(option_names->name), result.data.as_string);
+    case muesli_value_const_string:
+      ruby_set_table_string(cs, table, (char*)(option_names->name), result.data.as_const_string);
       break;
     case muesli_value_float:
       ruby_set_table_number(cs, table, (char*)(option_names->name), result.data.as_float);
@@ -256,8 +256,8 @@ muesli_ruby_eval_in_language()
     case muesli_value_integer:
       ruby_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      ruby_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      ruby_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       ruby_return_boolean(cs, result.data.as_bool);
@@ -296,8 +296,8 @@ ruby_eval_custom_function()
     case muesli_value_integer:
       ruby_return_integer(cs, result.data.as_int);
       break;
-    case muesli_value_string:
-      ruby_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      ruby_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_boolean:
       ruby_return_boolean(cs, result.data.as_int);
@@ -371,6 +371,7 @@ ruby_load_file(evaluator_interface *interface,
 		const char *filename)
 {
   int muesli_flags = interface->flags;
+  Muesli_Enter_Evaluator(interface, 0);
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loading %s\n", filename);
   }
@@ -378,6 +379,7 @@ ruby_load_file(evaluator_interface *interface,
   if (muesli_flags & TRACE_MUESLI_LOAD) {
     fprintf(stderr, "Loaded %s\n", filename);
   }
+  Muesli_Leave_Evaluator();
 }
 
 static muesli_value_t
@@ -387,6 +389,7 @@ ruby_eval_string(evaluator_interface *evaluator,
 		   int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(evaluator, transient);
   ANULL_VALUE(result);
 
   if (ruby_c_string) {
@@ -403,6 +406,7 @@ ruby_eval_string(evaluator_interface *evaluator,
     ambient_transient = old_ambient_transient;
   }
 
+  Muesli_Leave_Evaluator();
   return result;
 }
 
