@@ -21,7 +21,7 @@
 #ifndef _MUESLI_SIOD_
 #define _MUESLI_SIOD_
 
-#include "../config.h"
+#include "../muesli-config.h"
 
 // We only use plain SIOD if our own version of SIOD is not available.
 // This is because plain SIOD doesn't use a prefix for its names, and
@@ -156,8 +156,8 @@ get_parameter_siod(LISP option)
 		  1);
 
     switch (result_type) {
-    case muesli_value_string:
-      return strcons(strlen(result.as_string), result.as_string);
+    case muesli_value_const_string:
+      return strcons(strlen(result.as_const_string), result.as_const_string);
     case muesli_value_float:
       return flocons(result.as_float);
     case muesli_value_boolean:
@@ -197,9 +197,9 @@ get_parameters_siod()
 		    1);
 
       switch (result_type) {
-      case muesli_value_string:
-	if (result.as_string != NULL) {
-	  result = strcons(strlen(result.as_string), result.as_string);
+      case muesli_value_const_string:
+	if (result.as_const_string != NULL) {
+	  result = strcons(strlen(result.as_const_string), result.as_const_string);
 	} else {
 	  result = NIL;
 	}
@@ -415,13 +415,13 @@ siod_eval_string(const char *scratch,
 		 unsigned int string_length,
 		 eval_result_t *result_p)
 {
-  if (verbosity > standard) {
-    cout << "SIOD eval form: " << scratch << "\n";
-  }
-
   // Construct a lisp reader that will read from the scratch string
   const char *p;
   struct gen_readio s;
+  Muesli_Enter_Evaluator(, transient);
+  if (verbosity > standard) {
+    cout << "SIOD eval form: " << scratch << "\n";
+  }
   p = scratch;
   s.getc_fcn = (int (*)(void *))rfs_getc;
   s.ungetc_fcn = (void (*)(int,void *))rfs_ungetc;
@@ -437,9 +437,9 @@ siod_eval_string(const char *scratch,
   switch (TYPE(eval_result)) {
   case tc_string:
     if (result_p != NULL) {
-      *result_p->as_string = get_c_string(eval_result);
+      *result_p->as_const_string = get_c_string(eval_result);
     }
-    return muesli_value_string;
+    return muesli_value_const_string;
   case tc_nil:
     if (result_p != NULL) {
       *result_p->as_int = 0;
