@@ -21,7 +21,7 @@
 #ifndef _MUESLI_TCC_
 #define _MUESLI_TCC_
 
-#include "../config.h"
+#include "../muesli-config.h"
 
 #ifdef HAVE_LIBTCC
 // check with whether we're going to clash with SIOD over the symbol "load"
@@ -165,8 +165,8 @@ get_parameter_tcc(TCCState *cs)
 							   (char *)"tcc");
 
     switch (result.type) {
-    case muesli_value_string:
-      tcc_return_string(cs, result.data.as_string);
+    case muesli_value_const_string:
+      tcc_return_string(cs, result.data.as_const_string);
       break;
     case muesli_value_float:
       tcc_return_number(cs, result.data.as_float);
@@ -197,8 +197,8 @@ get_parameters_tcc(TCCState *cs)
 							   (char *)"tcc");
 
     switch (result.type) {
-    case muesli_value_string:
-      tcc_set_table_string(cs, table, (char*)(option_names->name), result.data.as_string);
+    case muesli_value_const_string:
+      tcc_set_table_string(cs, table, (char*)(option_names->name), result.data.as_const_string);
       break;
     case muesli_value_float:
       tcc_set_table_number(cs, table, (char*)(option_names->name), result.data.as_float);
@@ -321,8 +321,6 @@ static void
 tcc_load_file(evaluator_interface *interface,
 	      const char *filename)
 {
-  tcc_evaluator_init(interface);
-  
   int total_size = 1;
   char *place;
 
@@ -330,6 +328,9 @@ tcc_load_file(evaluator_interface *interface,
 
   int (*func)();
   unsigned long val;
+  Muesli_Enter_Evaluator(interface, transient);
+
+  tcc_evaluator_init(interface);
   
   if (stat(filename, &stat_buf) != 0) {
     // problem -- todo: handle this
@@ -374,6 +375,7 @@ tcc_load_file(evaluator_interface *interface,
     end_tcc_instance();
   }
 #endif
+  Muesli_Leave_Evaluator();
 }
 
 static muesli_value_t
@@ -383,11 +385,10 @@ tcc_eval_string(evaluator_interface *interface,
 		int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(interface, transient);
   ANULL_VALUE(result);
 
   if (tcc_c_string) {
-    int old_transient = ambient_transient;
-    ambient_transient = transient;
     // fprintf(stderr, "Tcc evaluating string \"%s\"\n", tcc_c_string);
     int (*func)();
     unsigned long val;
@@ -434,10 +435,10 @@ tcc_eval_string(evaluator_interface *interface,
 
     // Fill in: evaluate tcc_c_string, decide what kind of result it
     // returns, and fill in the result and its type accordingly.
-    ambient_transient = old_transient;
   }
   end_tcc_instance();
 
+  Muesli_Leave_Evaluator();
   return result;		// todo: get result type
 }
 
