@@ -20,7 +20,7 @@
 #ifndef _MUESLI_TCL_
 #define _MUESLI_TCL_
 
-#include "../config.h"
+#include "../muesli-config.h"
 
 #ifdef HAVE_LIBTCL
 
@@ -44,6 +44,25 @@ static evaluator_interface *tcl_interface = NULL;
 
 // static char *option_names_var_name = (char*)"option_names";
 
+Tcl_Obj *
+muesli_to_tcl(muesli_value_t mval)
+{
+  switch (mval.type) {
+  case muesli_value_const_string:
+    return Tcl_NewStringObj(mval.data.as_const_string,
+			    strlen(mval.data.as_const_string));
+  case muesli_value_malloced_string:
+    return Tcl_NewStringObj(mval.data.as_malloced_string,
+			    strlen(mval.data.as_malloced_string));
+  case muesli_value_float:
+    return Tcl_NewDoubleObj((double)mval.data.as_float);
+  case muesli_value_boolean:
+    return Tcl_NewBooleanObj(mval.data.as_int);
+  default:
+    return Tcl_NewBooleanObj(0);
+  }
+}
+
 static void
 set_one_parameter_tcl(char *name, char *value)
 {
@@ -121,35 +140,22 @@ get_parameter_tcl(ClientData clientData,
   if (objc != 2) {
     Tcl_WrongNumArgs(interp, 1, objv, "name");
     return TCL_ERROR;
-  }
+  } else {
 
-  char option_code = muesli_find_option_letter(tcl_interface->getopt_options, Tcl_GetString(objv[1]));
+    char option_code = muesli_find_option_letter(tcl_interface->getopt_options,
+						 Tcl_GetString(objv[1]));
 
-  if (option_code != -1) {
-    Tcl_Obj *result_obj;
-    muesli_value_t result = (tcl_interface->handle_option)(tcl_interface->app_params,
-							   option_code,	// option
-							   NULL, 0.0,	// value
-							   0,	// set
-							   (char *)"tcl");
+    if (option_code != -1) {
 
-    switch (result.type) {
-    case muesli_value_string:
-      result_obj = Tcl_NewStringObj(result.data.as_string, strlen(result.data.as_string));
-      break;
-    case muesli_value_float:
-      result_obj = Tcl_NewDoubleObj((double)result.data.as_float);
-      break;
-    case muesli_value_boolean:
-      result_obj = Tcl_NewBooleanObj(result.data.as_int);
-      break;
-    default:
-      result_obj = Tcl_NewBooleanObj(0);
-      break;
+      Tcl_SetObjResult(interp,
+		       muesli_to_tcl((tcl_interface->handle_option)(tcl_interface->app_params,
+								    option_code,	// option
+								    NULL, 0.0,	// value
+								    0,	// set
+								    (char *)"tcl")));
     }
-    Tcl_SetObjResult(interp, result_obj);
+    return TCL_OK;
   }
-  return TCL_OK;
 }
 
 static int
@@ -214,9 +220,9 @@ get_parameters_tcl(ClientData clientData,
 							   (char *)"tcl");
 
     switch (result.type) {
-    case muesli_value_string:
+    case muesli_value_const_string:
       STORE_ARY(option_names->name,
-		Tcl_NewStringObj(result.data.as_string, -1));
+		Tcl_NewStringObj(result.data.as_const_string, -1));
       break;
     case muesli_value_float:
       STORE_ARY(option_names->name,
@@ -244,6 +250,7 @@ get_parameters_tcl(ClientData clientData,
   return TCL_OK;
 }
 
+/* todo: use muesli_to_tcl instead */
 void
 muesli_to_tcl_result(Tcl_Interp *interp, muesli_value_t mval)
 {
@@ -262,10 +269,11 @@ muesli_to_tcl_result(Tcl_Interp *interp, muesli_value_t mval)
     Tcl_SetObjResult(interp,
 		     Tcl_NewIntObj(mval.data.as_int));
     break;
-  case muesli_value_string:
+  case muesli_value_malloced_string:
+  case muesli_value_const_string:
     Tcl_SetObjResult(interp,
-		     Tcl_NewStringObj(mval.data.as_string,
-				      strlen(mval.data.as_string)));
+		     Tcl_NewStringObj(mval.data.as_const_string,
+				      strlen(mval.data.as_const_string)));
     break;
   case muesli_value_boolean:
     Tcl_SetObjResult(interp,
@@ -384,14 +392,13 @@ tcl_eval_string(evaluator_interface *interface,
 		int transient)
 {
   muesli_value_t result;
+  Muesli_Enter_Evaluator(interface, transient);
   ANULL_VALUE(result);
 
   // Set the global one
   tcl_interpreter = (Tcl_Interp*)interface->state;
 
   if (tcl_c_string) {
-    int old_ambient_transient = ambient_transient;
-    ambient_transient = transient;
 
     // fprintf(stderr, "Tcl evaluating string \"%s\" with state %p\n", tcl_c_string, (Tcl_Interp*)interface->state);
     int result_code = Tcl_EvalEx((Tcl_Interp*)interface->state,
@@ -399,8 +406,6 @@ tcl_eval_string(evaluator_interface *interface,
 				 TCL_EVAL_DIRECT | TCL_EVAL_GLOBAL);
     if (result_code == TCL_OK) {
 
-      ambient_transient = old_ambient_transient;
-
       Tcl_Obj *result_obj = Tcl_GetObjResult(tcl_interpreter);
 
 #if 0
@@ -437,18 +442,18 @@ tcl_eval_string(evaluator_interface *interface,
 	    result.type = muesli_value_float;
 	    free(raw_string_result);
 	  } else { 
-	    result.data.as_string = raw_string_result;
-	    result.type = muesli_value_string;
+	    result.data.as_malloced_string = raw_string_result;
+	    result.type = muesli_value_malloced_string;
 	  }
 	}
     } else {
-      ambient_transient = old_ambient_transient;
       Tcl_Obj *error_obj = Tcl_GetObjResult(tcl_interpreter);
       char *error_message = Tcl_GetString(error_obj);
       fprintf(stderr, "Error from Tcl interpreter while interpreting: %s; code=%d, message %s\n", tcl_c_string, result_code, error_message);
       result.type = muesli_value_error;
     }
   }
+  Muesli_Leave_Evaluator();
   return result;
 }
 
@@ -463,6 +468,8 @@ tcl_setup(evaluator_interface *new_tcl_interface)
   tcl_interface->eval_string = tcl_eval_string;
   tcl_interface->load_file = tcl_load_file;
 
+  tcl_interface->from_muesli_value = (from_muesli_value_t)muesli_to_tcl;
+
   tcl_interface->version = TCL_PATCH_LEVEL;
 }
 
