@@ -21,7 +21,7 @@
 #ifndef _MUESLI_ZERO_
 #define _MUESLI_ZERO_
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include "muesli.h"
 #include "muesli-internal.h"
 
