@@ -1,4 +1,4 @@
-// muesli.c -*- C -*-
+/* muesli.c -*- C -*- */
 /* Interface to multiple interpreters.
    Copyright (C) 2008, 2009 University of Limerick
 
@@ -20,7 +20,7 @@
 #ifndef _MUESLI_C_
 #define _MUESLI_C_
 
-#include "../config.h"
+#include "../muesli-config.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -31,6 +31,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <limits.h>
+#include <math.h>
 
 #ifdef HAVE_LIBREADLINE
 #include <readline/readline.h>
@@ -237,10 +238,10 @@ muesli_get_evaluator_by_extension(const char *extension)
     if ((evaluator->extension != NULL)
 	&& strcmp(evaluator->extension, extension) == 0) {
       if (choice_1 == NULL) {
-	fprintf(stderr, "First possibility for %s is %s\n", extension, evaluator->implementation_name);
+	// fprintf(stderr, "First possibility for %s is %s\n", extension, evaluator->implementation_name);
 	choice_1 = evaluator;
       } else  {
-	fprintf(stderr, "Other possibility for %s is %s\n", extension, evaluator->implementation_name);
+	// fprintf(stderr, "Other possibility for %s is %s\n", extension, evaluator->implementation_name);
 	choice_2 = evaluator;
       }
     }
@@ -259,7 +260,7 @@ muesli_get_evaluator_by_extension(const char *extension)
 	for (;
 	     preferences != NULL;
 	     preferences = strchr(preferences, ',')) {
-	  fprintf(stderr, "Looking in \"%s\" for \"%s\"\n", preferences, name_2);
+	  // fprintf(stderr, "Looking in \"%s\" for \"%s\"\n", preferences, name_2);
 	  if (strncmp(preferences, name_2, name_2_len) == 0) {
 	    choice_1 = choice_2;
 	    break;
@@ -268,7 +269,7 @@ muesli_get_evaluator_by_extension(const char *extension)
       }
     }
 
-    // Make sure it's ready for use
+    /* Make sure it's ready for use */
     muesli_initialize_evaluator(choice_1);
 
     return choice_1;
@@ -342,7 +343,7 @@ muesli_result_type_name(value_type_tag_t enumerated)
   switch (enumerated) {
   case muesli_value_none: return (char*)"void";
   case muesli_value_float: return (char*)"number";
-  case muesli_value_string: return (char*)"string";
+  case muesli_value_const_string: return (char*)"string";
   case muesli_value_boolean: return (char*)"boolean";
   case muesli_value_error: return (char*)"error";
   default: return (char*)"unknown";
@@ -370,30 +371,244 @@ muesli_eval_in_language(const char *language,
 					    transient);
 }
 
+char *
+muesli_eval_to_malloced_string(evaluator_interface *ev,
+			       char *fragment,
+			       unsigned int fragment_length,
+			       int transient,
+			       int *success_flag_p)
+{
+#define NBUF_MAX 256
+  char nbuf[NBUF_MAX];
+  muesli_value_t mv = (ev->eval_string)(ev,
+					fragment, fragment_length,
+					transient);
+
+  if (success_flag_p != NULL) {
+    *success_flag_p = 0;
+  }
+
+  switch (mv.type) {
+  case muesli_value_none:
+    return NULL;
+  case muesli_value_float:
+    snprintf(nbuf, NBUF_MAX, "%g", mv.data.as_float);
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return muesli_malloc_copy_string(nbuf);
+  case muesli_value_integer:
+    snprintf(nbuf, NBUF_MAX, "%d", mv.data.as_int);
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return muesli_malloc_copy_string(nbuf);
+  case muesli_value_const_string:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return muesli_malloc_copy_string(mv.data.as_const_string);
+  case muesli_value_malloced_string:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return mv.data.as_malloced_string;
+  case muesli_value_boolean:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return muesli_malloc_copy_string((mv.data.as_bool) ? "true" : "false");
+  case muesli_value_error:
+    return NULL;
+  }
+  return NULL;
+}
+
+const char *
+muesli_eval_to_const_string(evaluator_interface *ev,
+		      char *fragment,
+		      unsigned int fragment_length,
+		      int transient,
+		      int *success_flag_p)
+{
+#define NBUF_MAX 256
+  static char nbuf[NBUF_MAX];
+  muesli_value_t mv = (ev->eval_string)(ev,
+					fragment, fragment_length,
+					transient);
+
+  if (success_flag_p != NULL) {
+    *success_flag_p = 0;
+  }
+
+  switch (mv.type) {
+  case muesli_value_none:
+    return NULL;
+  case muesli_value_float:
+    snprintf(nbuf, NBUF_MAX, "%g", mv.data.as_float);
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return nbuf;
+  case muesli_value_integer:
+    snprintf(nbuf, NBUF_MAX, "%d", mv.data.as_int);
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return nbuf;
+  case muesli_value_const_string:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return mv.data.as_const_string;
+  case muesli_value_malloced_string:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return mv.data.as_malloced_string;
+  case muesli_value_boolean:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return (mv.data.as_bool) ? (const char*)"true" : (const char*)"false";
+  case muesli_value_error:
+    return NULL;
+  }
+  return NULL;
+}
+
+float
+muesli_eval_to_float(evaluator_interface *ev,
+		     char *fragment,
+		     unsigned int fragment_length,
+		     int transient,
+		     int *success_flag_p)
+{
+  muesli_value_t mv = (ev->eval_string)(ev,
+					fragment, fragment_length,
+					transient);
+
+  if (success_flag_p != NULL) {
+    *success_flag_p = 0;
+  }
+
+  switch (mv.type) {
+  case muesli_value_none:
+    return nanf("none");
+  case muesli_value_float:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return mv.data.as_float;
+  case muesli_value_integer:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return (float)(mv.data.as_int);
+  case muesli_value_const_string:
+  case muesli_value_malloced_string:
+    if (success_flag_p != NULL) {
+      *success_flag_p = (isdigit(mv.data.as_const_string[0])
+			 || (mv.data.as_const_string[0] == '-'));
+    }
+    return atof(mv.data.as_const_string);
+  case muesli_value_boolean:
+    return mv.data.as_bool ? 1.0 : 0.0;
+  case muesli_value_error:
+    return nanf("error");
+  }
+  return 0.0;
+}
+
 int
-muesli_load_file(const char *filename)
+muesli_eval_to_int(evaluator_interface *ev,
+		   char *fragment,
+		   unsigned int fragment_length,
+		   int transient,
+		   int *success_flag_p)
+{
+  muesli_value_t mv = (ev->eval_string)(ev,
+					fragment, fragment_length,
+					transient);
+
+  if (success_flag_p != NULL) {
+    *success_flag_p = 0;
+  }
+
+  switch (mv.type) {
+  case muesli_value_none:
+    return 0;
+  case muesli_value_float:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return floor(mv.data.as_float);
+  case muesli_value_integer:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return mv.data.as_int;
+  case muesli_value_const_string:
+  case muesli_value_malloced_string:
+    if (success_flag_p != NULL) {
+      *success_flag_p = (isdigit(mv.data.as_const_string[0])
+			 || (mv.data.as_const_string[0] == '-'));
+    }
+    return atoi(mv.data.as_const_string);
+  case muesli_value_boolean:
+    return mv.data.as_bool ? 1 : 0;
+  case muesli_value_error:
+    return 0;
+  }
+
+  return 0;
+}
+
+int
+muesli_eval_to_boolean(evaluator_interface *ev,
+		       char *fragment,
+		       unsigned int fragment_length,
+		       int transient,
+		       int *success_flag_p)
+{
+  muesli_value_t mv = (ev->eval_string)(ev,
+					fragment, fragment_length,
+					transient);
+
+  if (success_flag_p != NULL) {
+    *success_flag_p = 0;
+  }
+
+  switch (mv.type) {
+  case muesli_value_none:
+    return 0;
+  case muesli_value_float:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return mv.data.as_float != 0.0;
+  case muesli_value_integer:
+    if (success_flag_p != NULL) { *success_flag_p = 1; }
+    return mv.data.as_int != 0;
+  case muesli_value_const_string:
+  case muesli_value_malloced_string:
+    if (strcmp(mv.data.as_const_string, "true")) {
+      if (success_flag_p != NULL) { *success_flag_p = 1; }
+      return 1;
+    }
+    if (strcmp(mv.data.as_const_string, "false")) {
+      if (success_flag_p != NULL) { *success_flag_p = 1; }
+      return 0;
+    }
+  case muesli_value_boolean:
+    return mv.data.as_bool;
+  case muesli_value_error:
+    return 0;
+  }
+
+  return 0;
+}
+
+int
+muesli_load_file(const char *filename, int silent)
 {
   const char *extension = strrchr(filename, '.');
   if (extension == NULL) {
-    fprintf(stderr,
-	    "Could not find extension of \"%s\" -- I do not know how to load this file\n",
-	    filename);
+    if (!silent) {
+      fprintf(stderr,
+	      "Could not find extension of \"%s\" -- I do not know how to load this file\n",
+	      filename);
+    }
     return 0;
   }
   extension++;
   if (*extension == '\0') {
-    fprintf(stderr,
-	    "Empty extension in \"%s\" -- I do not know how to load this file\n",
-	    filename);
+    if (!silent) {
+      fprintf(stderr,
+	      "Empty extension in \"%s\" -- I do not know how to load this file\n",
+	      filename);
+    }
     return 0;
   }
 
   evaluator_interface *interface = muesli_get_evaluator_by_extension(extension);
 
   if (interface == NULL) {
-    fprintf(stderr,
-	    "I could not work out how to load \"%s\" from the extension \"%s\"\n",
-	    filename, extension);
+    if (!silent) {
+      fprintf(stderr,
+	      "I could not work out how to load \"%s\" from the extension \"%s\"\n",
+	      filename, extension);
+    }
     return 0;
   }
 
@@ -401,23 +616,67 @@ muesli_load_file(const char *filename)
   return 1;
 }
 
-void
+void *
+muesli_map_interpreter_names(map_interpreter_names_function_t fn,
+			     void *val)
+{
+  evaluator_interface *evaluator;
+
+  for (evaluator = evaluator_interfaces;
+       evaluator != NULL;
+       evaluator = evaluator->next) {
+    val = fn(val, evaluator->implementation_name, evaluator->language_name);
+  }
+  return val;
+}
+
+int
 muesli_print_interpreter_names(FILE *stream,
 			       const char *format,
 			       const char *selected_name,
-			       const char *selected_flag)
+			       const char *selected_flag,
+			       unsigned int *widest_language_return,
+			       unsigned int *widest_implementation_return)
 {
   evaluator_interface *eval;
+  int n = 0;
+  int widest_language = 0;
+  int widest_implementation = 0;
+
+  if (format == NULL) {
+    format = "%s";
+  }
 
   for (eval = evaluator_interfaces;
        eval != NULL;
        eval = eval->next) {
-    fprintf(stream, format, eval->language_name, eval->implementation_name,
-	   (((selected_name != NULL)
-	     && (strcmp(eval->implementation_name, selected_name) == 0))
-	    ? (selected_flag ? selected_flag : " (selected)") : ""),
-	   eval->version);
+    int width = strlen(eval->language_name);
+    if (width > widest_language) {
+      widest_language = width;
+    }
+    width = strlen(eval->implementation_name);
+    if (width > widest_implementation) {
+      widest_implementation = width;
+    }
+    if (stream != NULL) {
+      fprintf(stream, format, eval->language_name, eval->implementation_name,
+	      (((selected_name != NULL)
+		&& (strcmp(eval->implementation_name, selected_name) == 0))
+	       ? (selected_flag ? selected_flag : " (selected)") : ""),
+	      eval->version);
+    }
+    n++;
   }
+
+  if (widest_language_return != NULL) {
+    *widest_language_return = widest_language;
+  }
+
+  if (widest_implementation_return != NULL) {
+    *widest_implementation_return = widest_implementation;
+  }
+
+  return n;
 }
 
 /*******************/
@@ -426,45 +685,21 @@ muesli_print_interpreter_names(FILE *stream,
 
 evaluator_interface *no_evaluator = NULL; // might put in a novelty one to print a message
 
-evaluator_interface *scripting_evaluator = NULL;
+evaluator_interface *current_muesli_evaluator = NULL;
 
 evaluator_interface *evaluator_common_lisp = NULL;
 evaluator_interface *evaluator_custom = NULL;
-evaluator_interface *evaluator_ecl = NULL;
-evaluator_interface *evaluator_elk = NULL;
-evaluator_interface *evaluator_erlang = NULL;
-evaluator_interface *evaluator_external_exec = NULL;
 evaluator_interface *evaluator_external_pipe = NULL;
-evaluator_interface *evaluator_gs = NULL;
-evaluator_interface *evaluator_guile = NULL;
-evaluator_interface *evaluator_haskell = NULL;
-evaluator_interface *evaluator_javascript = NULL;
-evaluator_interface *evaluator_jvm = NULL;
-evaluator_interface *evaluator_ksi = NULL;
-evaluator_interface *evaluator_lua = NULL;
 evaluator_interface *evaluator_machine = NULL;
-evaluator_interface *evaluator_matheval = NULL;
-evaluator_interface *evaluator_octave = NULL;
-evaluator_interface *evaluator_parrot = NULL;
-evaluator_interface *evaluator_perl = NULL;
-evaluator_interface *evaluator_prolog = NULL;
-evaluator_interface *evaluator_python = NULL;
-evaluator_interface *evaluator_R = NULL;
-evaluator_interface *evaluator_ruby = NULL;
-evaluator_interface *evaluator_siod = NULL;
-evaluator_interface *evaluator_slang = NULL;
-evaluator_interface *evaluator_smalltalk = NULL;
-evaluator_interface *evaluator_stack_code = NULL;
-evaluator_interface *evaluator_tcc = NULL;
-evaluator_interface *evaluator_tcl = NULL;
 evaluator_interface *evaluator_zero = NULL;
 
-/* You should use set_evaluator_app_specializer(string,
+/* You should use muesli_set_evaluator_app_specializer(string,
    specializer_function) to set a function to add your extensions.
    This is called when the interpreter is initialized, which happens
-   when you call get_named_evaluator, so you must call
-   set_evaluator_app_specializer before then.
+   when you call muesli_get_named_evaluator, so you must call
+   muesli_set_evaluator_app_specializer before then.
 */
+
 void
 muesli_set_evaluator_app_specializer(char *evaluator_name,
 				     evaluator_specializer_t specializer)
@@ -485,9 +720,11 @@ muesli_set_evaluator_app_specializer(char *evaluator_name,
   }
 }
 
-// This is an internal function; we call it for you.
-// set_evaluator_app_specializer is the one you should call to set
-// this up.
+/* This is an internal function; we call it for you.
+   muesli_set_evaluator_app_specializer is the one you should call to set
+   this up.
+*/
+
 void
 muesli_app_specialize(char *name,
 	       evaluator_interface *interface)
@@ -497,7 +734,9 @@ muesli_app_specialize(char *name,
   }
 }
 
-// A wrapper for new evaluator_interface, that registers the name in the map evaluators_by_name
+/* A wrapper for muesli_make_evaluator_interface, that registers the
+   name in the map evaluators_by_name
+*/
 evaluator_interface *
 muesli_define_evaluator(const char *new_language_name,
 			const char *new_implementation_name,
@@ -571,13 +810,14 @@ muesli_register_evaluators(void *new_app_params,
   muesli_app_env = app_env;
 
   muesli_extensions = extensions;
-  // These create the interfaces, and give them the information
-  // they'll need when the interpreters initialize.  These do not
-  // initialize the interpreters; that happens later.
-  evaluator_external_exec = muesli_define_evaluator("exec", NULL, "sh", 0,
-						    external_exec_setup, exec_evaluator_init,
-						    params_handler, getopt_options,
-						    new_app_params, new_app_data);
+  /* These create the interfaces, and give them the information
+     they'll need when the interpreters initialize.  These do not
+     initialize the interpreters; that happens later.
+  */
+  muesli_define_evaluator("exec", NULL, "sh", 0,
+			  external_exec_setup, exec_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
   evaluator_external_pipe = muesli_define_evaluator("pipe", NULL, "", 0,
 						    external_pipe_setup, pipe_evaluator_init,
 						    params_handler, getopt_options,
@@ -591,150 +831,151 @@ muesli_register_evaluators(void *new_app_params,
 					   params_handler, getopt_options,
 					   new_app_params, new_app_data);
 #ifdef HAVE_LIBECL
-  evaluator_ecl = muesli_define_evaluator("Common Lisp", "ecl", "lisp", 0,
-					  ecl_setup, ecl_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
+  muesli_define_evaluator("Common Lisp", "ecl", "lisp", 0,
+			  ecl_setup, ecl_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBELK
-  evaluator_elk = muesli_define_evaluator("scheme", "elk", "scm", 0,
-					  elk_setup, elk_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
+  muesli_define_evaluator("scheme", "elk", "scm", 0,
+			  elk_setup, elk_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBERL_INTERFACE
-  evaluator_erlang = muesli_define_evaluator("erlang", "erlang", "er", 0,
-					     erlang_setup, erlang_evaluator_init,
-					     params_handler, getopt_options,
-					     new_app_params, new_app_data);
+  muesli_define_evaluator("erlang", "erlang", "er", 0,
+			  erlang_setup, erlang_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBGS
-  evaluator_gs = muesli_define_evaluator("PostScript", "gs", "ps", 0,
-					 gs_setup, gs_evaluator_init,
-					 params_handler, getopt_options,
-					 new_app_params, new_app_data);
+  muesli_define_evaluator("PostScript", "gs", "ps", 0,
+			  gs_setup, gs_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBGUILE
-  evaluator_guile = muesli_define_evaluator("scheme", "guile", "scm", 0,
-					    guile_setup, guile_evaluator_init,
-					    params_handler, getopt_options,
-					    new_app_params, new_app_data);
+  muesli_define_evaluator("scheme", "guile", "scm", 0,
+			  guile_setup, guile_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBKSI
-  evaluator_ksi = muesli_define_evaluator("scheme", "ksi", "scm", 0,
-					  ksi_setup, ksi_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
+  muesli_define_evaluator("scheme", "ksi", "scm", 0,
+			  ksi_setup, ksi_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBMATHEVAL
-  evaluator_matheval = muesli_define_evaluator("infix", "matheval", "math", 0,
-					       matheval_setup, matheval_evaluator_init,
-					       params_handler, getopt_options,
-					       new_app_params, new_app_data);
+  muesli_define_evaluator("infix", "matheval", "math", 0,
+			  matheval_setup, matheval_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBMOZJS
-  evaluator_javascript = muesli_define_evaluator("javascript", "SpiderMonkey", "js", 0,
-						 javascript_setup, javascript_evaluator_init,
-						 params_handler, getopt_options,
-						 new_app_params, new_app_data);
+  muesli_define_evaluator("javascript", "SpiderMonkey", "js", 0,
+			  javascript_setup, javascript_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBJVM
-  evaluator_jvm = muesli_define_evaluator("jvm", "jvm", "class", 1,
-					  jvm_setup, jvm_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
+  muesli_define_evaluator("jvm", "jvm", "class", 1,
+			  jvm_setup, jvm_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBLUA
-  evaluator_lua = muesli_define_evaluator("lua", NULL, "lua", 0,
-					  lua_setup, lua_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
+  muesli_define_evaluator("lua", NULL, "lua", 0,
+			  lua_setup, lua_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBOCTINTERP
-  evaluator_octave = muesli_define_evaluator("octave", NULL, "oct", 0,
-					     octave_setup, octave_evaluator_init,
-					     params_handler, getopt_options,
-					     new_app_params, new_app_data);
+  muesli_define_evaluator("octave", NULL, "oct", 0,
+			  octave_setup, octave_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBPARROT
-  evaluator_parrot = muesli_define_evaluator("parrot", NULL, "peg", 1,
-					     parrot_setup, parrot_evaluator_init,
-					     params_handler, getopt_options,
-					     new_app_params, new_app_data);
+  muesli_define_evaluator("parrot", NULL, "peg", 1,
+			  parrot_setup, parrot_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBPERL
-  evaluator_perl = muesli_define_evaluator("perl", NULL, "pl", 0,
-					   perl_setup, perl_evaluator_init,
-					   params_handler, getopt_options,
-					   new_app_params, new_app_data);
+  muesli_define_evaluator("perl", NULL, "pl", 0,
+			  perl_setup, perl_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBPL
-  evaluator_prolog = muesli_define_evaluator("prolog", "SWI Prolog", ".pl", 0,
-					     prolog_setup, prolog_evaluator_init,
-					     params_handler, getopt_options,
-					     new_app_params, new_app_data);
+  muesli_define_evaluator("prolog", "SWI Prolog", ".pl", 0,
+			  prolog_setup, prolog_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #if defined(HAVE_LIBPYTHON) && defined(HAVE_PYTHON2_6_PYTHON_H)
-  evaluator_python = muesli_define_evaluator("python", "CPython", "py", 0,
-					     python_setup, python_evaluator_init,
-					     params_handler, getopt_options,
-					     new_app_params, new_app_data);
+  muesli_define_evaluator("python", "CPython", "py", 0,
+			  python_setup, python_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBR
-  evaluator_R = muesli_define_evaluator("R", NULL, "r", 0,
-					R_setup, R_evaluator_init,
-					params_handler, getopt_options,
-					new_app_params, new_app_data);
+  muesli_define_evaluator("R", NULL, "r", 0,
+			  R_setup, R_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBRUBY_STATIC
-  evaluator_ruby = muesli_define_evaluator("ruby", NULL, "rb", 0,
-					   ruby_setup, ruby_evaluator_init,
-					   params_handler, getopt_options,
-					   new_app_params, new_app_data);
+  muesli_define_evaluator("ruby", NULL, "rb", 0,
+			  ruby_setup, ruby_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #if defined(HAVE_LIBSIOD) || defined(HAVE_LIBULSIOD)
-  evaluator_siod = muesli_define_evaluator("scheme", "siod", "scm", 0,
-					   siod_setup, siod_evaluator_init,
-					   params_handler, getopt_options,
-					   new_app_params, new_app_data);
+  muesli_define_evaluator("scheme", "siod", "scm", 0,
+			  siod_setup, siod_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBSLANG
-  evaluator_slang = muesli_define_evaluator("s-lang", NULL, "sl", 0,
-					    slang_setup, slang_evaluator_init,
-					    params_handler, getopt_options,
-					    new_app_params, new_app_data);
+  muesli_define_evaluator("s-lang", NULL, "sl", 0,
+			  slang_setup, slang_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBSYX
-  evaluator_smalltalk = muesli_define_evaluator("smalltalk", "syx", "st", 0,
-						syx_setup, syx_evaluator_init,
-						params_handler, getopt_options,
-						new_app_params, new_app_data);
+  muesli_define_evaluator("smalltalk", "syx", "st", 0,
+			  syx_setup, syx_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBTCC
-  evaluator_tcc = muesli_define_evaluator("Tiny C", "tcc", "c", 0,
-					  tcc_setup, tcc_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
+  muesli_define_evaluator("Tiny C", "tcc", "c", 0,
+			  tcc_setup, tcc_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBTCL
-  evaluator_tcl = muesli_define_evaluator("tcl", NULL, "tcl", 0,
-					  tcl_setup, tcl_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
+  muesli_define_evaluator("tcl", NULL, "tcl", 0,
+			  tcl_setup, tcl_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
 #endif
   evaluator_machine = muesli_define_evaluator("machine", NULL, "o", 1,
 					      machine_code_setup, machine_code_evaluator_init,
 					      params_handler, getopt_options,
 					      new_app_params, new_app_data);
-  evaluator_stack_code = muesli_define_evaluator("stack-code", NULL, "sc", 1,
-						 stack_code_setup, stack_bytecode_evaluator_init,
-						 params_handler, getopt_options,
-						 new_app_params, new_app_data);
+  muesli_define_evaluator("stack-code", NULL, "sc", 1,
+			  stack_code_setup, stack_bytecode_evaluator_init,
+			  params_handler, getopt_options,
+			  new_app_params, new_app_data);
   {
     /* The Lisp evaluator is configured dynamically, rather than
        through autoconf; we look around to see what's available. */
     const char *common_lisp = getenv((char*)"COMMON_LISP");
     const char *executable = NULL;
+    evaluator_interface *evaluator_common_lisp = NULL;
     if (common_lisp == NULL) {
       /* todo: look for different Lisps on the path */
       common_lisp = (char*)"sbcl";
@@ -772,13 +1013,13 @@ muesli_register_evaluators(void *new_app_params,
       haskell = (char*)"hugs";
     }
     /* todo: set these parameters for the specific implementation found */
-    evaluator_haskell = muesli_define_variant_evaluator(evaluator_external_pipe,
-							"haskell",
-							"hugs",
-							haskell,
-							"Type :? for help\nHugs> ",
-							"Hugs> ",
-							":quit\n");
+    muesli_define_variant_evaluator(evaluator_external_pipe,
+				    "haskell",
+				    "hugs",
+				    haskell,
+				    "Type :? for help\nHugs> ",
+				    "Hugs> ",
+				    ":quit\n");
     /* todo: what is Haskell's array syntax? */
   }
 }
@@ -845,8 +1086,9 @@ muesli_evaluator_repl(const char *evaluator_name)
 	case muesli_value_integer:
 	  printf("%d\n", result.data.as_int);
 	  break;
-	case muesli_value_string:
-	  printf("\"%s\"\n", result.data.as_string);
+	case muesli_value_const_string:
+	case muesli_value_malloced_string:
+	  printf("\"%s\"\n", result.data.as_const_string);
 	  break;
 	case muesli_value_boolean:
 	  printf("%s\n", result.data.as_bool ? "true" : "false");
@@ -885,8 +1127,9 @@ muesli_split_args(const char *string, const char *string0, char ***new_argv, int
 				+ 1);
   char *q = scratch;
 
-  // count the possible breaks between arguments -- possibly an
-  // overestimate as some may be quoted, but that doesn't matter
+  /* count the possible breaks between arguments -- possibly an
+     overestimate as some may be quoted, but that doesn't matter
+  */
   while ((c = *p++) != '\0') {
     switch (c) {
     case ' ': case '\t': case '\n': case '\r': n_parts++; break;
@@ -957,7 +1200,7 @@ muesli_split_path()
   char *scratch = (char*)malloc(strlen(string) + 1);
   char *q = scratch;
 
-  // count the path element separators
+  /* count the path element separators */
   while ((c = *p++) != '\0') {
     if (c == ':') {
       n_parts++;
@@ -1007,23 +1250,18 @@ muesli_find_executable(const char *basename)
 	strcat(prog_name, "/");
       }
       strcat(prog_name, basename);
-      // fprintf(stderr, "Trying \"%s\"\n", prog_name);
       if (stat(prog_name, &statbuf) == 0) {
-	// fprintf(stderr, "Statted %s\n", prog_name);
 	if (statbuf.st_mode & S_IXOTH) {
-	  // fprintf(stderr, "OK by world\n");
 	  break;
 	}
 	if ((statbuf.st_mode & S_IXUSR)
 	    && ((statbuf.st_uid == getuid())
 		|| (statbuf.st_uid == geteuid())))
 	  {
-	    // fprintf(stderr, "OK by user\n");
 	    break;
 	  }
 	if (statbuf.st_mode & S_IXGRP) {
 	  if (statbuf.st_gid == getgid()) {
-	    // fprintf(stderr, "OK by main group\n");
 	    break;
 	  } else {
 	    gid_t gids[NGROUPS_MAX];
@@ -1032,7 +1270,6 @@ muesli_find_executable(const char *basename)
 	      int j;
 	      for (j = 0; j < n_extra_groups; j++) {
 		if (statbuf.st_gid == gids[j]) {
-		  fprintf(stderr, "OK by other group\n");
 		  break;
 		}
 	      }
@@ -1076,8 +1313,6 @@ muesli_set_parameter(evaluator_interface *interface,
   char option_code = muesli_find_option_letter(options, option);
   option_handler_t handler = interface->handle_option;
 
-  // fprintf(stderr, "option_code=%c (%d)\n", option_code, option_code);
-
   if (option_code != -1) {
     (handler)(interface->app_params, option_code,
 	      value, 0.0, 1, language);
@@ -1087,7 +1322,8 @@ muesli_set_parameter(evaluator_interface *interface,
   }
 }
 
-static char num_buf[64];
+#define NUM_BUF_SIZE 64
+static char num_buf[NUM_BUF_SIZE];
 
 const char*
 muesli_get_parameter(evaluator_interface *interface,
@@ -1098,20 +1334,20 @@ muesli_get_parameter(evaluator_interface *interface,
   char option_code = muesli_find_option_letter(options, option);
   if (option_code != -1) {
     muesli_value_t result = (handler)(interface->app_params,
-				      option_code,	// option
-				      NULL, 0.0,	// value
-				      0,	// set
+				      option_code,	/* option */
+				      NULL, 0.0,	/* value */
+				      0,		/* set */
 				      NULL);
 
     switch (result.type) {
-    case muesli_value_string:
-      return result.data.as_string;
+    case muesli_value_const_string:
+      return result.data.as_const_string;
     case muesli_value_float:
-      sprintf(num_buf, "%f", result.data.as_float);
-      return num_buf;
+      snprintf(num_buf, NUM_BUF_SIZE, "%f", result.data.as_float);
+      return muesli_malloc_copy_string(num_buf);
     case muesli_value_integer:
-      sprintf(num_buf, "%d", result.data.as_int);
-      return num_buf;
+      snprintf(num_buf, NUM_BUF_SIZE, "%d", result.data.as_int);
+      return muesli_malloc_copy_string(num_buf);
     case muesli_value_boolean:
       if (result.data.as_bool) {
 	return "true";
